/**
 * @(#)InGuiTraversal.java  6 November 2006
 *
 * @author Viera K. Proulx
 */
import java.io.BufferedReader;

import edu.neu.ccs.*;
import edu.neu.ccs.gui.*;
import edu.neu.ccs.util.*;

/** 
 * Functional (External) iterator for a list of cities 
 * to generate data from the user input through a GUI panel.
 */
class InGuiTraversal implements Traversal {
  
  /*-------------------------------------------------------- 
   Member data
   *------------------------------------------------------*/
  /** The display to install in the general dialog. */
  protected Display cDisplay;
  
  /** The view for one city. */
  protected CityView cView; 
  
  /** The general dialog for city input. */
  protected GeneralDialog input;
  
  /** A <CODE>{@link City City}</CODE> object buffer 
   * to hold data for one city at a time. 
   */
  protected City c = new City();
  
  /** Determines whether the general dialog has been closed. */
  protected boolean closed = true;
  
  /** Determines whether new city data has been submitted. */
  protected boolean submitted = false;
  
  /** The submit action. */
  protected SimpleAction submitAction =
    new SimpleAction("Submit") {
    public void perform() { submitAction(); }
  };
  
  /** The cancel action. */
  protected SimpleAction cancelAction =
    new SimpleAction("Cancel") {
    public void perform() { cancelAction(); }
  };
  
  /*-------------------------------------------------------- 
   Constructor
   *------------------------------------------------------*/
  /**
   * The constructor.
   * 
   * <p>Create the view for the input of city data and place it
   * in a nonmodal dialog.</p>
   * <p>read the data to initialize the 
   * <CODE>{@link City City}</CODE> buffer.</p>
   */
  public InGuiTraversal() { 
    
    this.cView = new CityView();    
    this.cDisplay = cView.cView;
    
    /* create non-modal dialog for the city data input */
    input = 
      new GeneralDialog
      (cDisplay,
          "City Input",
          new Object[][]{{"Submit", submitAction, DialogAction.KEEP_OPEN},
          {"Cancel", cancelAction, DialogAction.AUTO_CLOSE}},
          false);
    
    /* specify the closing action */
    input.setWindowClosingAction(cancelAction, DialogAction.AUTO_CLOSE);
    
    /* currently not closed */
    closed = false;   
    input.setVisible(true); 
    
    /* extract values for the first city */
    getRest();
  }
  
  /*--------------------------------------------------------
   Methods to implement the Traversal interface
   *------------------------------------------------------*/
  
  /**
   * Nullify current data in the city buffer and busy wait 
   * for next submission.
   * On exit - either new data is available in the city buffer
   * or the state indicates the end of input. 
   */
  public synchronized void getnext() {
    
    submitted = false;
    while ((!submitted)&&(!closed)) JPTUtilities.pauseThread(10);
    if (closed)
      input.dispose();
  }
  
  /**
   * <p>Request the input GUI to initialize the
   *  <CODE>{@link City City}</CODE> object buffer with the next data.</p>
   *  <p>Indicate the <em>end of input</em> if user aborts the request.</p>
   * @see Traversal#getRest()
   */
  public Traversal getRest() {
    getnext();
    return this;
  }
  
  /**
   * <p>Produce the city buffer object.</p>
   * <p>Report an error if the end of file has been reached.</p>
   * @see Traversal#getFirst()
   */
  public synchronized Object getFirst() {
    
    if ((submitted) && (!closed))
      return c;
    
    else
      return null;
  }
  
  /**
   * The end of input predicate.  
   *
   *@return true when the user has indicated the end of data.
   * @see Traversal#isEmpty()
   */
  public synchronized boolean isEmpty() {
    return !((submitted) && (!closed)); 
  }
  
  
  /**
   * Use the GUI input data to set the city buffer values.
   * Record the success in the <CODE>submitted</CODE> tag.
   */
  protected void submitAction(){
    c = cView.getCity();
    submitted = true;
  }
  
  /** 
   * Record the user's end of input in the <CODE>closed</CODE> tag.
   */
  protected void cancelAction(){
    closed = true;
  }
}

